////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.flickr;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.watto.program.android.sync.flickr.authenticator.FlickrAuthenticator;


/***********************************************************************************************
 Provides utility methods for communicating with the backend server
***********************************************************************************************/
public class FlickrQuery {

  /***********************************************************************************************
  Fetches the list of friend data updates from the server
  @return list The list of updates received from the server.
  ***********************************************************************************************/
  public static FlickrFriend[] getFriends(){
    try {

      if (!FlickrAuthenticator.credentialsKnown()) {
        // The authentication is invalid, so return nothing
        return new FlickrFriend[0];
      }

      // Get details about my friends
      JSONObject friendsObject = queryServer("http://api.flickr.com/services/rest/?method=flickr.contacts.getList");
      JSONArray friendArray = friendsObject.getJSONObject("contacts").getJSONArray("contact");

      int friendCount = friendArray.length();
      FlickrFriend[] friends = new FlickrFriend[friendCount];

      // Create the friends
      for (int i = 0;i < friendCount;i++) {
        friends[i] = new FlickrFriend(friendArray.getJSONObject(i));
      }

      // get further details about each friend
      getFriendsDetails(friends);

      return friends;

    }
    catch (Throwable t) {
      // An error occurred in the response, or there were parsing errors
      return new FlickrFriend[0];
    }
  }


  /***********************************************************************************************
  Fetches further details for a list of <code>FoursquareFriend</code>s from the server
  @param friends The friends being queried
  ***********************************************************************************************/
  public static void getFriendsDetails(FlickrFriend[] friends){
    try {

      int friendCount = friends.length;
      for (int i = 0;i < friendCount;i++) {
        FlickrFriend friend = friends[i];

        // get the userID
        String userId = friends[i].getUserID();
        if (userId == null || userId.equals("")) {
          // user ID not valid, so skip the user
          continue;
        }

        // get the details for this user
        JSONObject detailsObject = queryServer("http://api.flickr.com/services/rest/?method=flickr.people.getInfo&user_id=" + userId);
        detailsObject = detailsObject.getJSONObject("person");

        // Gender
        friend.setGender(detailsObject.optString("gender",""));

        // Photo, generated from the iconServer and iconFarm
        friend.setPhoto(detailsObject.optLong("iconserver",0),detailsObject.optLong("iconfarm",0));

        // Location (Home City)
        try {
          friend.setHomeCity(detailsObject.getJSONObject("location").optString("_content",""));
        }
        catch (JSONException e){
          // Ignore where the location object doesn't exist
        }

        // Photos Object
        try {
          JSONObject photosObject = detailsObject.getJSONObject("photos").getJSONObject("count");
          friend.setPhotoCount(photosObject.optLong("_content",0));

        }
        catch (JSONException e) {
          // Ignore where the photos object doesn't exist
        }

      }

    }
    catch (Throwable t) {
      // Error in the response - maybe the server is down or the response message has changed
    }
  }


  /***********************************************************************************************
  Signs a query, sends it to the server, receives a response and converts it to a <code>JSONObject</code> 
  @param queryUrl the URL of the query you wish to perform
  @return the response as a <code>JSONObject</code>, or <b>null</b> if there was an error
  ***********************************************************************************************/
  public static JSONObject queryServer(String queryUrl){
    try {

      // Check that we have valid authentication
      if (!FlickrAuthenticator.credentialsKnown()) {
        return null;
      }

      // Sign the request with the access token
      queryUrl = FlickrAuthenticator.signRequest(queryUrl);
      HttpGet request = new HttpGet(queryUrl);

      // Send the request
      DefaultHttpClient client = new DefaultHttpClient();
      HttpResponse response = client.execute(request);

      // Get the response code
      if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
        // Success
        String responseMessage = EntityUtils.toString(response.getEntity());

        // Crop off the "jsonFlickrApi(" response header
        int bracketPosition = responseMessage.indexOf('(') + 1;
        if (bracketPosition <= 1 || responseMessage.length() < bracketPosition){
          return null;
        }
        responseMessage = responseMessage.substring(bracketPosition);
        
        // return the JSONObject for the response
        return new JSONObject(responseMessage);
      }
      return null;
    }
    catch (Throwable t) {
      return null;
    }
  }

}
