////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.flickr.authenticator;

import java.net.HttpURLConnection;
import java.net.URL;
import org.watto.android.component.WSWebView;
import org.watto.program.android.sync.flickr.GlobalConstants;
import org.watto.program.android.sync.flickr.security.FlickrAuthProtocol;
import org.watto.program.android.sync.flickr.sync.SyncIntervalWrapper;
import android.accounts.Account;
import android.accounts.AccountManager;
import android.app.AlarmManager;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.provider.ContactsContract;


/***********************************************************************************************
 Provides utility methods for communicating with the backend server
***********************************************************************************************/
public class FlickrAuthenticator {

  /** The Consumer Key, Consumer Secret, and callback URL for this application **/
  public static final FlickrAuthProtocol authClient = new FlickrAuthProtocol("b2805b96e43b8ccc9858c343347aa020","2ff9f345dbffd173",GlobalConstants.CALLBACK_URL);


  /***********************************************************************************************
  Authenticates the <i>username</i> and <i>password</i> on the backend server
  ***********************************************************************************************/
  public static boolean confirmCredentials(){
    try {

      if (!credentialsKnown()) {
        return false;
      }

      // The URL is the details of "myself"
      URL authURL = new URL(authClient.signRequest("http://api.flickr.com/services/rest/?method=flickr.test.login"));
      HttpURLConnection request = (HttpURLConnection)authURL.openConnection();

      // Send the request
      request.connect();

      // Get the response code
      if (request.getResponseCode() == 200) {
        return true;
      }
      return false;

    }
    catch (Throwable t) {
      return false;
    }
  }


  /***********************************************************************************************
  Creates a new <code>Account</code> and configures automatic syncing
  @param accountManager the <code>AccountManager</code> used to create the <code>Account</code>
  ***********************************************************************************************/
  public static Account createAccount(AccountManager accountManager){
    Account account = new Account(GlobalConstants.ACCOUNT_NAME,GlobalConstants.ACCOUNT_TYPE);

    // Create the account with the code as the password
    accountManager.addAccountExplicitly(account,authClient.getCode(),null);
    // Add the authentication token
    accountManager.setAuthToken(account,GlobalConstants.AUTHTOKEN_TYPE,authClient.getAccessToken());

    // Enable the auto-syncing of the account
    ContentResolver.setSyncAutomatically(account,ContactsContract.AUTHORITY,true);
    // Trigger the first sync immediately
    ContentResolver.requestSync(account,ContactsContract.AUTHORITY,new Bundle());

    return account;
  }


  /***********************************************************************************************
  Creates a new <code>Account</code> and configures automatic syncing (Android <= 2.1)
  @param accountManager the <code>AccountManager</code> used to create the <code>Account</code>
  @param alarmManager the <code>AlarmManager</code> used to schedule repeating events
  ***********************************************************************************************/
  public static void createAccount(AccountManager accountManager,AlarmManager alarmManager,Context context){
    createAccount(accountManager);

    // Repeat the sync every X minutes
    SyncIntervalWrapper.addPeriodicSync(context,GlobalConstants.DEFAULT_SYNC_INTERVAL);
  }


  /***********************************************************************************************
  Creates a new <code>Account</code> and configures automatic syncing (Android 2.2+)
  @param accountManager the <code>AccountManager</code> used to create the <code>Account</code>
  @param resolver the <code>ContentResolver</code> used to change the <code>Account</code> settings
  ***********************************************************************************************/
  public static void createAccount(AccountManager accountManager,ContentResolver resolver){
    Account account = createAccount(accountManager);

    // Repeat the sync every X minutes
    SyncIntervalWrapper.addPeriodicSync(resolver,account,ContactsContract.AUTHORITY,null,GlobalConstants.DEFAULT_SYNC_INTERVAL);
  }


  /***********************************************************************************************
  Gets whether the token and secret are specified or not
  ***********************************************************************************************/
  public static boolean credentialsKnown(){
    String accessToken = authClient.getAccessToken();
    if (accessToken == null || accessToken.equals("")) {

      // Try to load the access token from the Settings
      authClient.loadFromSettings();

      // Re-check the access token
      accessToken = authClient.getAccessToken();
      if (accessToken == null || accessToken.equals("")) {
        return false;
      }
    }
    return true;
  }


  public static String getAccessTokenUrl(String pinCode){
    return authClient.getAccessTokenUrl(pinCode);
  }


  /***********************************************************************************************
  Gets a <code>Bundle</code> for the account, containing the name, type, access token and code
  ***********************************************************************************************/
  public static Bundle getAccountBundle(AccountManager accountManager){
    Bundle bundle = new Bundle();
    bundle.putString(AccountManager.KEY_ACCOUNT_NAME,GlobalConstants.ACCOUNT_NAME);
    bundle.putString(AccountManager.KEY_ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
    bundle.putString(AccountManager.KEY_AUTHTOKEN,authClient.getAccessToken());
    bundle.putString(AccountManager.KEY_PASSWORD,authClient.getCode());
    return bundle;
  }


  /***********************************************************************************************
  Gets an <code>Intent</code> for the account, containing the name, type, access token and code
  ***********************************************************************************************/
  public static Intent getAccountIntent(AccountManager accountManager){
    Intent intent = new Intent();
    intent.putExtra(AccountManager.KEY_ACCOUNT_NAME,GlobalConstants.ACCOUNT_NAME);
    intent.putExtra(AccountManager.KEY_ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
    intent.putExtra(AccountManager.KEY_AUTHTOKEN,authClient.getAccessToken());
    intent.putExtra(AccountManager.KEY_PASSWORD,authClient.getCode());
    return intent;
  }


  /***********************************************************************************************
  Connects the user to the authentication URL. The user signs in and approves this application for
  use with the backend server. The token and secrets are generated by the backend server, and
  stored locally for signing HTTP requests.
  ***********************************************************************************************/
  public static Intent getAuthenticationIntent(Context context){
    try {
      // Generate the authentication URL
      String authURL = authClient.getAuthenticationUrl();

      // Generate the "show in browser" Intent
      Intent intent = new Intent(context,WSWebView.class);
      intent.putExtra("url",authURL);
      return intent;
    }
    catch (Throwable t) {
      return null;
    }
  }


  public static void retreiveAccessToken(String code){
    authClient.retreiveAccessToken(code);
  }


  public static String signRequest(String url){
    return authClient.signRequest(url);
  }


  /***********************************************************************************************
  Updates an existing <code>Account</code> with new credentials
  ***********************************************************************************************/
  public static void updateAccount(AccountManager accountManager){
    Account account = new Account(GlobalConstants.ACCOUNT_NAME,GlobalConstants.ACCOUNT_TYPE);
    accountManager.setAuthToken(account,GlobalConstants.AUTHTOKEN_TYPE,authClient.getAccessToken());
    accountManager.setPassword(account,authClient.getCode());
  }

}
