////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.flickr.security;

import java.io.IOException;
import java.security.MessageDigest;
import java.util.Arrays;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpParams;
import org.apache.http.util.EntityUtils;
import org.json.JSONException;
import org.json.JSONObject;
import org.watto.android.Settings;


/***********************************************************************************************
Convenience methods for establishing OAuth 2.0 authentication to Foursquare
***********************************************************************************************/
public class FlickrAuthProtocol {

  /** The OAuth key for this application **/
  String consumerKey = "";
  /** The OAuth secret for this application **/
  String consumerSecret = "";
  /** The callback URL for this application **/
  String callbackUrl = "watto://callback";
  /** The authentication code **/
  String code = "";
  /** The access token **/
  String accessToken = "";


  /***********************************************************************************************
  Initialises the OAuth provider with the <i>consumerKey</i>, <i>consumerSecret</i> and the 
  <i>callbackUrl</i> for the application
  @param consumerKey the OAuth key for this application
  @param consumerSecret the OAuth secret for this application
  @param callbackUrl the OAuth callback URL for this application
  ***********************************************************************************************/
  public FlickrAuthProtocol(String consumerKey, String consumerSecret, String callbackUrl){
    this.consumerKey = consumerKey;
    this.consumerSecret = consumerSecret;
    this.callbackUrl = callbackUrl;
  }


  /***********************************************************************************************
  Calculates the MD5 hash for a <i>message</i> and returns it in Hex format
  @param message the <code>String</code> to calculate the hash for
  @return the MD5 hash in Hex format
  ***********************************************************************************************/
  public String calculateMD5(String message){
    try {
      // Create the MD5 Hash
      MessageDigest digest = java.security.MessageDigest.getInstance("MD5");
      digest.update(message.getBytes());
      byte messageDigest[] = digest.digest();

      // Convert the hash to a Hex String      
      String hexString = "";
      for (int i = 0;i < messageDigest.length;i++) {
        String hexCharacter = Integer.toHexString(255 & messageDigest[i]);
        while (hexCharacter.length() < 2)
          hexCharacter = "0" + hexCharacter;
        hexString += hexCharacter;
      }

      return hexString;
    }
    catch (Throwable t) {
      return "";
    }
  }


  /***********************************************************************************************
  Gets the access token
  @return the <i>accessToken</i>
  ***********************************************************************************************/
  public String getAccessToken(){
    return accessToken;
  }


  /***********************************************************************************************
  Gets the URL for converting an authentication <i>code</i> into an access token
  @param code the authentication code
  @return the URL used to generate the access token
  ***********************************************************************************************/
  public String getAccessTokenUrl(String code){
    setCode(code);
    String url = "http://api.flickr.com/services/rest/?method=flickr.auth.getToken&format=json&api_key=" + consumerKey + "&frob=" + code;
    return signRequest(url,false);
  }


  /***********************************************************************************************
  Gets the URL for authenticating the application
  @return the URL used to authenticate the application
  ***********************************************************************************************/
  public String getAuthenticationUrl(){
    String url = "http://flickr.com/services/auth/?api_key=" + consumerKey + "&perms=read";
    return signRequest(url,false);
  }


  /***********************************************************************************************
  Gets the callback URL
  @return the <i>callbackUrl</i>
  ***********************************************************************************************/
  public String getCallbackUrl(){
    return callbackUrl;
  }


  /***********************************************************************************************
  Gets the authentication code
  @return the <i>code</i>
  ***********************************************************************************************/
  public String getCode(){
    return code;
  }


  /***********************************************************************************************
  Gets the OAuth key for this application
  @return the <i>consumerKey</i>
  ***********************************************************************************************/
  public String getConsumerKey(){
    return consumerKey;
  }


  /***********************************************************************************************
  Gets the OAuth secret for this application
  @return the <i>consumerSecret</i>
  ***********************************************************************************************/
  public String getConsumerSecret(){
    return consumerSecret;
  }


  /***********************************************************************************************
  Loads the authentication <i>code</i> and the <i>accessToken</i> from the <code>Settings</code>
  ***********************************************************************************************/
  public void loadFromSettings(){
    // Load the code and access token from the Settings
    if (Settings.hasApplication()) {
      this.code = Settings.getString("OAuth2Code");
      this.accessToken = Settings.getString("OAuth2AccessToken");
    }
  }


  /***********************************************************************************************
  Converts the authentication <i>code</i> into an access token
  @param code the authentication code
  @return the access token
  ***********************************************************************************************/
  public String retreiveAccessToken(String code){

    String tokenURL = getAccessTokenUrl(code);

    // Build a HttpClient that ignores SSL errors
    SchemeRegistry schemeRegistry = new SchemeRegistry();
    schemeRegistry.register(new Scheme("http",PlainSocketFactory.getSocketFactory(),80));
    schemeRegistry.register(new Scheme("https",new EasySSLSocketFactory(),443));

    HttpParams params = new BasicHttpParams();
    ClientConnectionManager clientConnectionManager = new ThreadSafeClientConnManager(params,schemeRegistry);

    DefaultHttpClient client = new DefaultHttpClient(clientConnectionManager,params);
    HttpPost post = new HttpPost(tokenURL);

    // Call the URL
    HttpResponse response = null;
    try {
      response = client.execute(post);
    }
    catch (ClientProtocolException e) {}
    catch (IOException e) {}

    String token = "";
    if (response == null || response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
      // error
      token = "";
    }
    else {
      // success - got a token
      try {
        String responseMessage = EntityUtils.toString(response.getEntity());

        // Crop off the "jsonFlickrApi(" response header
        int bracketPosition = responseMessage.indexOf('(') + 1;
        if (bracketPosition <= 1 || responseMessage.length() < bracketPosition) {
          return token;
        }
        responseMessage = responseMessage.substring(bracketPosition);

        JSONObject apiObject = new JSONObject(responseMessage);
        JSONObject accessTokenObject = apiObject.getJSONObject("auth").getJSONObject("token");
        token = accessTokenObject.getString("_content");
      }
      catch (ParseException e) {}
      catch (IOException e) {}
      catch (JSONException e) {}
    }

    setAccessToken(token);
    return token;

  }


  /***********************************************************************************************
  Sets the access token, and stores it in the <code>Settings</code>
  @param accessToken the access token
  ***********************************************************************************************/
  public void setAccessToken(String accessToken){
    this.accessToken = accessToken;

    // Store the access token in the Settings
    if (Settings.hasApplication()) {
      Settings.set("OAuth2AccessToken",accessToken);
    }
  }


  /***********************************************************************************************
  Sets the callback URL
  @param callbackUrl the callback URL
  ***********************************************************************************************/
  public void setCallbackUrl(String callbackUrl){
    this.callbackUrl = callbackUrl;
  }


  /***********************************************************************************************
  Sets the authentication code, and stores it in the <code>Settings</code>
  @param code the authentication code
  ***********************************************************************************************/
  public void setCode(String code){
    this.code = code;

    // Store the code in the Settings
    if (Settings.hasApplication()) {
      Settings.set("OAuth2Code",code);
    }
  }


  /***********************************************************************************************
  Sets the OAuth key for this application
  @param consumerKey the OAuth key
  ***********************************************************************************************/
  public void setConsumerKey(String consumerKey){
    this.consumerKey = consumerKey;
  }


  /***********************************************************************************************
  Sets the OAuth secret for this application
  @param consumerSecret the OAuth secret
  ***********************************************************************************************/
  public void setConsumerSecret(String consumerSecret){
    this.consumerSecret = consumerSecret;
  }


  /***********************************************************************************************
  Signs a URL request with the api_sig and api_key
  @param url the request to sign
  @return the signed request
  ***********************************************************************************************/
  public String signRequest(String url){
    return signRequest(url,true);
  }


  /***********************************************************************************************
  Signs a URL request with the api_sig and optionally the api_key
  @param url the request to sign
  @param attachKey whether the api_key attribute should be attached to the URL or not
  @return the signed request
  ***********************************************************************************************/
  public String signRequest(String url,boolean attachKey){

    int questionMarkPosition = url.indexOf('?');

    if (attachKey) {
      // Add the format and api_key arguments
      if (questionMarkPosition > 0) {
        // already has a '?', so need a '&' instead
        url += "&";
      }
      else {
        url += "?";
        questionMarkPosition = url.indexOf('?');
      }
      url += "format=json&api_key=" + consumerKey + "&auth_token=" + accessToken;
    }

    String hash = "";

    // 1. Get all the arguments of the URL
    hash = url.substring(questionMarkPosition + 1);
    String[] arguments = hash.split("&");

    // 2. Sort the arguments alphabetically
    Arrays.sort(arguments);

    // 3. Concatenate the consumerSecret and all the name-value arguments
    hash = consumerSecret;

    int argumentCount = arguments.length;
    for (int i = 0;i < argumentCount;i++) {
      String argument = arguments[i];
      hash += argument.replace("=","");
    }

    // 4. Calculate the MD5 hash of the string
    hash = calculateMD5(hash);

    // 5. Append the hash to the URL
    return url + "&api_sig=" + hash;

  }

}
